// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_multimin.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <i.h>
#include <util.h>
#include "../../server.h"
#include "../../sim.h"
#include "fit.h"
#include "patch.h"

static int unused __attribute__((unused));

static char config_files[3000];

void fit_set_light(double in)
{
edit_file("light.inp","",2,in);
}

void fit_set_volt_start(double in)
{
edit_file("jv.inp","",2,in);
}

void fit_set_sim_mode(int in)
{
edit_file_int("sim.inp","",2,in);
}

void fit_set_external_volt(double in)
{
edit_file("sim.inp","",32,in);
}

void fit_set_shg_voltage(double in)
{
edit_file("shg.inp","",14,in);
edit_file("shg.inp","",28,in);
}

void fit_set_dt(double in)
{
edit_file("tpc.inp","",6,in);
}

void fit_set_rep_rate(int in)
{
edit_file_int("tpc.inp","",20,in);
}

void fit_set_pulse_len(int in)
{
edit_file_int("tpc.inp","",18,in);
}

void fit_set_shg_laser_on(int in)
{
edit_file_int("shg.inp","",30,in);
}


void fit_build_job(struct server *myserver,char *base_name,int i)
{
char tempsim[1000];
char tempexp[1000];
char command[2000];
char patch_file_path[1000];

	sprintf(tempsim,"./sim/%s%d",base_name,i);
	sprintf(tempexp,"./exp/%s%d",base_name,i);
	mkdir(tempsim, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

	strcat(tempexp,"/");
	strcat(tempsim,"/");

	sprintf(command,"cp %s %s",config_files,tempsim);
	unused=system(command);
	if (get_dump_status(dump_iodump)==TRUE) printf("copy command = %s\n",command);
	sprintf(patch_file_path,"%s/patch.inp",tempexp);
	patch(tempsim,patch_file_path);

	server_add_job(myserver,tempsim);
}

double fit_run_sims(struct fitvars *fitconfig,struct server *myserver)
{
if (get_dump_status(dump_iodump)==TRUE) printf("fit_run_sims\n");
double error=0.0;
char tempsim[1000];
char tempexp[1000];
char command[2000];
char tempstring[1000];

  DIR           *d;
  struct dirent *dir;
  d = opendir(".");
strcpy(config_files,"");
  if (d)
  {
    while ((dir = readdir(d)) != NULL)
    {
     if (strcmp_end(dir->d_name,".inp")==0)
	{
		strcat(config_files,dir->d_name);
		strcat(config_files," ");
	}
    }

    closedir(d);
  }
//printf("copy files= %s\n",config_files);

strcpy(config_files,config_files);
double voltage;
int i;

gen_dos_fd_gaus_fd();

//getchar();
//printf("about to run for dark\n");
mkdir("./sim", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

for (i=0;i<fitconfig->fit_jv;i++)
{
	printf("Submitting JV job %d\n",i);
	fit_build_job(myserver,"jv",i);
}

for (i=0;i<fitconfig->stark_diff;i++)
{
	printf("Submitting stark zero field job  %d\n",i);
	fit_build_job(myserver,"stark_zero",i);

}

for (i=0;i<fitconfig->stark_max;i++)
{
	printf("Submitting stark job %d/%d\n",i,fitconfig->stark_max);
	fit_build_job(myserver,"stark",i);
}

for (i=0;i<fitconfig->pulse_max;i++)
{
	printf("Submitting pulse job %d/%d\n",i,fitconfig->pulse_max);
	fit_build_job(myserver,"pulse",i);
}

for (i=0;i<fitconfig->pulse_voc_max;i++)
{
	printf("Submitting pulse voc job %d/%d\n",i,fitconfig->pulse_voc_max);
	fit_build_job(myserver,"pulse_voc",i);
}

for (i=0;i<fitconfig->starknp_max;i++)
{
	printf("Running time starknp %d/%d\n",i,fitconfig->stark_max);
	fit_build_job(myserver,"starknp",i);
}

for (i=0;i<fitconfig->tpv_max;i++)
{

	printf("Submitting TPV job %d/%d\n",i,fitconfig->stark_max);
	fit_build_job(myserver,"tpv",i);
}

for (i=0;i<fitconfig->tpcmax;i++)
{
	printf("Submitting TPC job %d/%d\n",i,fitconfig->tpcmax);
	fit_build_job(myserver,"tpc",i);
}

for (i=0;i<fitconfig->celiv_max;i++)
{
	printf("Submitting CELIV job %d/%d\n",i,fitconfig->celiv_max);
	fit_build_job(myserver,"celiv",i);
}

for (i=0;i<fitconfig->tofmax;i++)
{
	printf("Submitting ToF job %d/%d\n",i,fitconfig->tofmax);
	fit_build_job(myserver,"tof",i);
}

for (i=0;i<fitconfig->cemax;i++)
{
	printf("Submitting CE job %d/%d\n",i,fitconfig->tofmax);
	fit_build_job(myserver,"ce",i);
}

int ce_dark_max=0;
if (fitconfig->dotimece==TRUE)
{
	struct istruct exp;
	inter_load_a(&exp,"./exp/cedata.dat",0,1);
	double start=exp.x[0];
	double stop=exp.x[exp.len-1];
	double steps=3.0;
	double step=(stop-start)/steps;
	//printf("%e %e\n",start,stop);
	//getchar();
	inter_free(&exp);
	double V=start;

	do
	{
	printf("Running time ce %d\n",ce_dark_max);
	sprintf(tempsim,"./sim/ce%d",ce_dark_max);
	sprintf(tempexp,"./exp/ce%d",ce_dark_max);
	mkdir(tempsim, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

	strcat(tempexp,"/");
	strcat(tempsim,"/");

	fit_set_light(0.0);					//light off
	fit_set_volt_start(0.05);				//change vstart
	fit_set_sim_mode(2);					//set simulation mode
	fit_set_dt(2.4000e-06);					//set dt
	fit_set_rep_rate(25);					//set rep rate
	fit_set_pulse_len(4);					//pulse length
	fit_set_external_volt(V);				//set external voltage
	sprintf(command,"cp %s %s",config_files,tempsim);
	unused=system(command);

	server_add_job(myserver,tempsim);

	V+=step;
	ce_dark_max++;
	}while(ce_dark_max<=steps);


	//run_simulation(tempsim);
}


int ce_light_max=0;
if (fitconfig->dotimecelight==TRUE)
{
	struct istruct exp;
	inter_load_a(&exp,"./exp/celight.dat",0,1);
	double start=exp.x[0];
	double stop=exp.x[exp.len-1];
	double steps=3.0;
	double step=(stop-start)/steps;
	//printf("%e %e\n",start,stop);
	//getchar();
	inter_free(&exp);
	double V=start;

	do
	{
	printf("Running time ce light %d\n",ce_light_max);
	sprintf(tempsim,"./sim/celight%d",ce_light_max);
	sprintf(tempexp,"./exp/celight%d",ce_light_max);
	mkdir(tempsim, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

	strcat(tempexp,"/");
	strcat(tempsim,"/");

	fit_set_light(1000.0);					//One sun
	fit_set_volt_start(0.05);		//change vstart
	fit_set_sim_mode(2);					//set simulation mode
	fit_set_dt(2.4000e-06);					//set dt
	fit_set_rep_rate(25);					//set rep rate
	fit_set_pulse_len(4);					//pulse length
	fit_set_external_volt(V);				//set external voltage
	sprintf(command,"cp %s %s",config_files,tempsim);
	unused=system(command);

	server_add_job(myserver,tempsim);

	V+=step;
	ce_light_max++;
	}while(ce_light_max<=steps);


	//run_simulation(tempsim);
}
//fitconfig->tpclightmax



for (i=0;i<fitconfig->shg_max;i++)
{
char tempsim_on[100];
char tempsim_off[100];

	printf("Running time shg %d/%d\n",i,fitconfig->shg_max);
	sprintf(tempsim_on,"./sim/shg_on_%d",i);
	sprintf(tempsim_off,"./sim/shg_off_%d",i);

	sprintf(tempexp,"./exp/shg%d",i);
	mkdir(tempsim_on, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	mkdir(tempsim_off, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);


	strcat(tempexp,"/");
	strcat(tempsim_on,"/");
	strcat(tempsim_off,"/");

	FILE *info=fopena(tempexp,"info.inp","r");
	unused=fscanf(info,"%s",tempstring);
	unused=fscanf(info,"%le",&voltage);
	unused=fscanf(info,"%s",tempstring);

	if (strcmp("#end",tempstring)!=0)
	{
		printf("Problem with %sinfo.inp\n",tempexp);
		exit(0);
	}
	fclose(info);

	printf("Doing shg %d %lf\n",i,voltage);
	fit_set_light(0.0);					//light bias
	fit_set_sim_mode(24);					//set simulation mode
	fit_set_shg_voltage(voltage);				//set externalv

	fit_set_shg_laser_on(0);

	sprintf(command,"cp %s %s",config_files,tempsim_off);
	unused=system(command);

	fit_set_shg_laser_on(1);
	sprintf(command,"cp %s %s",config_files,tempsim_on);
	unused=system(command);

	server_add_job(myserver,tempsim_on);
	server_add_job(myserver,tempsim_off);
}


print_jobs(myserver);
server_run_jobs(myserver);
change_cpus(myserver);
//getchar();
if (fitconfig->dotimece==TRUE)
{
	mkdir("./sim/ce", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	FILE *r;
	r=fopen("./sim/ce/realce.dat","w");
	fclose(r);
	double nread=0.0;
	double V=0.0;
	for (i=0;i<ce_dark_max;i++)
	{
		sprintf(tempsim,"./sim/ce%d/cej.dat",i);
		r=fopen(tempsim,"r");
		unused=fscanf(r,"%le %le",&V,&nread);
		fclose(r);
		r=fopen("./sim/ce/realce.dat","a");
		fprintf(r,"%le %le\n",V,nread);
		fclose(r);
	}
}

if (fitconfig->dotimecelight==TRUE)
{
	mkdir("./sim/celight", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	FILE *r;
	r=fopen("./sim/celight/realce.dat","w");
	fclose(r);
	double nread=0.0;
	double V=0.0;
	for (i=0;i<ce_light_max;i++)
	{
		sprintf(tempsim,"./sim/celight%d/cej.dat",i);
		r=fopen(tempsim,"r");
		unused=fscanf(r,"%le %le",&V,&nread);
		fclose(r);
		r=fopen("./sim/celight/realce.dat","a");
		fprintf(r,"%le %le\n",V,nread);
		fclose(r);
	}
}
error=get_all_error(fitconfig);
printf("Jobs/s=%le ODEs/s=%le\n",server_get_jobs_per_s(),server_get_odes_per_s());
return error;
}

