# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import pygtk
pygtk.require('2.0')
import gtk
import sys
import os
import shutil
import signal
import subprocess
from tempfile import mkstemp

class my_data():
	token=""
	units=""
	info=""
	def __init__(self,a,b,c):
		self.token=a
		self.units=b
		self.info=c

class tokens:
	lib=[]
	lib.append(my_data("#mueffe","m<sup>2</sup>V<sup>-1</sup>s<sup>-1</sup>","Electron mobility"))
	lib.append(my_data("#mueffh","m<sup>2</sup>V<sup>-1</sup>s<sup>-1</sup>","Hole mobility"))
	lib.append(my_data("#Ntrape","m<sup>-3</sup> eV<sup>-1</sup>","Electron trap density"))
	lib.append(my_data("#Ntraph","m<sup>-3</sup> eV<sup>-1</sup>","Hole trap density"))
	lib.append(my_data("#Etrape","eV","Electron tail slope"))
	lib.append(my_data("#Etraph","eV","Hole tail slope"))
	lib.append(my_data("#epsilonr","none","Relative permittivity"))
	lib.append(my_data("#srhsigman_e","m<sup>-2</sup>","Free electron->Trapped electron"))
	lib.append(my_data("#srhsigmap_e","m<sup>-2</sup>","Trapped electron->Free hole"))
	lib.append(my_data("#srhsigman_h","m<sup>-2</sup>","Trapped hole->Free electron"))
	lib.append(my_data("#srhsigmap_h","m<sup>-2</sup>","Free hole->Trapped hole"))
	lib.append(my_data("#Rshunt","Ohms","Shunt resistance"))
	lib.append(my_data("#Rcontact","Ohms","Series resistance"))
	lib.append(my_data("#lcharge","m<sup>-3</sup>","Charge on left contact"))
	lib.append(my_data("#rcharge","m<sup>-3</sup>","Charge on right contact"))
	lib.append(my_data("#Vstart","V","Start voltage"))
	lib.append(my_data("#Vstop","V","Stop voltage"))
	lib.append(my_data("#Vstep","V","Voltage step"))
	lib.append(my_data("#Psun","0/1000","Intensity of the sun"))
	lib.append(my_data("#simplephotondensity","m<sup>-2</sup>s<sup>-1</sup>","Photon density"))
	lib.append(my_data("#simple_alpha","m<sup>-1</sup>","Absorption of material"))
	lib.append(my_data("#plot","1/0","Plot bands etc.. "))
	lib.append(my_data("#celiv_laser_eff","1/0","#celiv_laser_eff"))
	lib.append(my_data("#doping","m<sup>-3</sup>","#doping"))
	lib.append(my_data("#simmode","au","#simmode"))
	lib.append(my_data("#Nc","m<sup>-3</sup>","Effective density of free electron states"))
	lib.append(my_data("#Nv","m<sup>-3</sup>","Effective density of free hole states"))
	lib.append(my_data("#maxelectricalitt","au","Max electical itterations"))
	lib.append(my_data("#electricalclamp","au","Electrical clamp"))
	lib.append(my_data("#posclamp","au","Poisson clamping"))
	lib.append(my_data("#electricalerror","au","Minimum electrical error"))
	lib.append(my_data("#free_to_free_recombination","m<sup>3</sup>s<sup>-1</sup>","Free to free recombination"))
	lib.append(my_data("#sun","filename","Sun's spectra"))
	lib.append(my_data("#meshpoints","au","Mesh points (x)"))
	lib.append(my_data("#lpoints","au","Mesh points (lambda)"))
	lib.append(my_data("#lstart","m","Lambda start"))
	lib.append(my_data("#lstop","m","Lambda stop"))
	lib.append(my_data("#laserwavelength","m","Laser wavelength"))
	lib.append(my_data("#spotx","m","#spotx"))
	lib.append(my_data("#spoty","m","#spoty"))
	lib.append(my_data("#pulseJ","J","#pulseJ"))
	lib.append(my_data("#pulseL","s","#pulseL"))
	lib.append(my_data("#gather","au","#gather"))
	lib.append(my_data("#dlength","m","#dlength"))
	lib.append(my_data("#electron_eff","0-1","#electron_eff"))
	lib.append(my_data("#hole_eff","0-1","#hole_eff"))
	lib.append(my_data("#function","au","#function"))
	lib.append(my_data("#lr_pcontact","left/right","P contact on left or right"))
	lib.append(my_data("#Vexternal","V","start voltage"))
	lib.append(my_data("#Vmax","V","Max voltage"))
	lib.append(my_data("#celiv_t0","s","#celiv_t0"))
	lib.append(my_data("#celiv_t1","s","#celiv_t1"))
	lib.append(my_data("#celiv_t2","s","#celiv_t2"))
	lib.append(my_data("#Eg","eV","Eg"))
	lib.append(my_data("#Xi","eV","Xi"))
	lib.append(my_data("#start_stop_time","s","Time of pause"))
	lib.append(my_data("#stopstart","1/0","Pause between iterations"))
	lib.append(my_data("#invert_current","true/false","Invert output"))
	lib.append(my_data("#lr_bias","left/right","Bias left or right"))
	lib.append(my_data("#otherlayers","m","Other layers"))
	lib.append(my_data("#dump_slices","1/0","Dump slices"))
	lib.append(my_data("#tofstop","s","ToF stop time"))
	lib.append(my_data("#toflaseroff","s","ToF laser off time"))
	lib.append(my_data("#dt","s","ToF time step"))
	lib.append(my_data("#tofdtmull","au","ToF dt multiplier"))
	lib.append(my_data("#toflaserpower","1/0","ToF laser power"))
	lib.append(my_data("#start_steps","1/0","ToF steps before start"))
	lib.append(my_data("#tof_voltage","V","ToF voltage"))
	lib.append(my_data("#tof_Rshort","Ohms","Rshort"))
	lib.append(my_data("#stark_startime","s","startime"))
	lib.append(my_data("#stark_ea_factor","au","ea_factor"))
	lib.append(my_data("#stark_Np","1/0","Np"))
	lib.append(my_data("#stark_den","1/0","den"))
	lib.append(my_data("#stark_externalv","V","externalv"))
	lib.append(my_data("#stark_dt_neg_time","s","dt_neg_time"))
	lib.append(my_data("#stark_dt","s","dt"))
	lib.append(my_data("#stark_dt_mull","au","dt_mull"))
	lib.append(my_data("#stark_stop","s","stop"))
	lib.append(my_data("#stark_stark","1/0","stark"))
	lib.append(my_data("#stark_lasereff","1/0","lasereff"))
	lib.append(my_data("#stark_probe_wavelength","m","wavelength"))
	lib.append(my_data("#stark_sim_contacts","1/0","sim_contacts"))
	lib.append(my_data("#pulse_dt","s","dt"))
	lib.append(my_data("#pulse_t0","s","time<sub>0</sub>"))
	lib.append(my_data("#pulse_t1","s","time<sub>1</sub>"))
	lib.append(my_data("#pulse_t2","s","time<sub>2</sub>"))
	lib.append(my_data("#pulse_Vexternal","Volts","V<sub>external</sub>"))
	lib.append(my_data("#pulse_repeat","number","Repeat"))
	lib.append(my_data("#Rshort_pulse","Ohms","R<sub>short</sub>"))
	lib.append(my_data("#pulse_laser_on","s","laser_on"))
	lib.append(my_data("#pulse_laser_width","s","laser_width"))
	lib.append(my_data("#pulse_laser_power","J","laser_power"))
	lib.append(my_data("#pulse_light","1/0","pulse_light"))
	lib.append(my_data("#pulse_sun","J","pulse_sun"))
	lib.append(my_data("#pulse_shift","s","pulse_shift"))
	lib.append(my_data("#pulse_bigmeshstart","s","Big mesh start"))
	lib.append(my_data("#pulse_bigmeshstop","s","Big mesh stop"))
	lib.append(my_data("#pulse_bigdt","s","Big mesh dt"))
	lib.append(my_data("#photokit_points","s","points"))
	lib.append(my_data("#photokit_n","s","Wavelengths to simulate"))
	lib.append(my_data("#photokit_Vexternal","Volts","External voltage"))
	lib.append(my_data("#Rshort_photokit","Ohms","R<sub>short</sub>"))
	lib.append(my_data("#photokit_sun","1000=1 Sun","Backgroud light bias"))
	lib.append(my_data("#photokit_modulation_max","1000=1 Sun","Modulation depth"))
	lib.append(my_data("#photokit_modulation_fx","Hz","Modulation frequency"))
	lib.append(my_data("#Rshort_pulse_voc","Ohms","R short"))
	lib.append(my_data("#pulse_voc_laser_eff","0-1.0","Laser Efficiency"))
	lib.append(my_data("#pulse_voc_light_bias","0-1000","Light bias"))
	lib.append(my_data("#Dphotoneff","0-1","Photon efficiency"))
	lib.append(my_data("#jv_step_mul","0-2.0","JV voltage step multiplyer"))
	lib.append(my_data("#dump_slices_by_time","1/0","dump slices by time"))
	lib.append(my_data("#dump_all_slices","1/0","dump all slices"))
	lib.append(my_data("#voc","V","V<sub>oc</sub>"))

	lib.append(my_data("#photokit_r","au","Re - Amps"))
	lib.append(my_data("#photokit_i","au","Im - Amps"))
	lib.append(my_data("#photokit_fx","Hz","Frequency"))

	lib.append(my_data("#simplephotondensity","m<sup>-2</sup>s<sup>-1</sup>","Photon Flux"))
	lib.append(my_data("#simple_alpha","m<sup>-1</sup>","Absorption"))


	def find(self,token):
		for i in range(0, len(self.lib)):
			if self.lib[i].token==token.strip():
				return self.lib[i]
		#print "Not found",token
		return False
