// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include "dump.h"
#include <math.h>

void init_dump(struct device *in)
{
int i=0;
if (get_dump_status(dump_iodump)==TRUE)
{
FILE *out;
	out=fopena(in->outputpath,"./init_Fi.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e %e\n",in->ymesh[i],in->Fi[i]);
	}
	fclose(out);

	out=fopena(in->outputpath,"./init_Ec.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e %e\n",in->ymesh[i],in->Ec[i]);
	}
	fclose(out);

	out=fopena(in->outputpath,"./init_Ev.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e %e\n",in->ymesh[i],in->Ev[i]);
	}
	fclose(out);

	out=fopena(in->outputpath,"./init_n.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e %e\n",in->ymesh[i],in->n[i]);
	}
	fclose(out);

	out=fopena(in->outputpath,"./init_p.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
		fprintf(out,"%e %e\n",in->ymesh[i],in->p[i]);
	}
	fclose(out);
}
}

void my_guess(struct device *in)
{
int i;

//getchar();
double Ef=0.0;
double Eg=in->Eg[0];
double Xi=in->Xi[0];
double charge_left=in->lcharge;
double charge_right=in->rcharge;

double top_l=0.0;
double top_r=0.0;
if (in->interfaceleft==TRUE)
{
	top_l=in->phibleft-Eg;
}else
{
#ifdef enable_define_contact
	if (in->lr_pcontact==LEFT)
	{
#endif
		top_l=get_top_from_p(charge_left,in->Te[0],in->imat[0]);
#ifdef enable_define_contact
	}else
	{
		top_l=-(Eg+get_top_from_n(charge_left,in->Te[0],in->imat[0]));
	}
#endif
}

if (in->interfaceright==TRUE)
{
	top_r=-in->phibright;
}else
{
#ifdef enable_define_contact
	if (in->lr_pcontact==LEFT)
	{
#endif
		top_r=get_top_from_n(charge_right,in->Te[in->ymeshpoints-1],in->imat[in->ymeshpoints-1]);
#ifdef enable_define_contact
	}else
	{
		top_r=-(Eg+get_top_from_p(charge_right,in->Te[in->ymeshpoints-1],in->imat[in->ymeshpoints-1]));
	}
#endif
}
if (get_dump_status(dump_iodump)==TRUE)
{
printf("check1= %e %e\n",get_p_den(top_l,in->Te[0],in->imat[0]),charge_left);
printf("check2= %e %e\n",get_n_den(top_r,in->Te[in->ymeshpoints-1],in->imat[in->ymeshpoints-1]),charge_right);
}

double delta_phi=top_l+top_r+in->Eg[0]+in->Xi[0]-in->Xi[in->ymeshpoints-1];
double test_l=-in->Xi[0]+top_r;
double test_r=-in->Xi[0]-in->Eg[0]-top_l;
in->vbi=delta_phi;
if (get_dump_status(dump_print_text)==TRUE)
{
printf("delta=%le\n",delta_phi);
printf(">>>>top_l= %le\n",top_l+Eg);
printf(">>>>top_r= %le\n",-top_r);
printf("left= %le right = %le  %le %le\n",test_l,test_r,test_r-test_l,delta_phi);
printf("%le %le %le %le %le\n",top_l,top_r,Eg,delta_phi,in->phi[0]);
}

Ef=-(top_l+Xi+Eg);

if (get_dump_status(dump_iodump)==TRUE)
{
double Lp=get_p_den((-Xi-in->phi[0]-Eg)-Ef,in->Th[0],in->imat[0]);
double Ln=get_n_den(Ef-(-Xi-in->phi[0]),in->Te[0],in->imat[0]);
double Rp=get_p_den((-Xi-delta_phi-Eg)-Ef,in->Th[in->ymeshpoints-1],in->imat[in->ymeshpoints-1]);
double Rn=get_n_den(Ef-(-Xi-delta_phi),in->Te[in->ymeshpoints-1],in->imat[in->ymeshpoints-1]);
printf("Ef=%e\n",Ef);
printf ("Lp = %e\n", Lp);
printf ("Ln = %e\n", Ln); //,5e26*exp(q*(Ef-(-Xi-in->phi[0]))/(300.0*kb))

printf ("Rp = %e\n", Rp);
printf ("Rn = %e\n", Rn);
FILE *contacts=fopena(in->outputpath,"./contacts.dat","w");
fprintf (contacts,"%le\n", Lp);
fprintf (contacts,"%le\n", Ln);

fprintf (contacts,"%le\n", Rp);
fprintf (contacts,"%le\n", Rn);
fclose(contacts);
}

//getchar();
for (i=0;i<in->ymeshpoints;i++)
{

	in->phi[i]=0;
	in->Ec[i]=-in->phi[i]-in->Xi[i];
	in->Ev[i]=-in->phi[i]-in->Xi[i]-in->Eg[i];

	in->Fi[i]=Ef;

	in->Fn[i]=Ef;
	in->Fp[i]=Ef;
	if (in->Fi[i]<in->Ev[i]) in->Fi[i]=in->Ev[i];
	//if (in->Fi[i]<in->Ev[i]) in->Fi[i]=in->Ec[i];

	double t=in->Fi[i]-in->Ec[i];
	double tp=in->Ev[i]-in->Fi[i];

	in->n[i]=in->Nc[i]*exp(((t)*q)/(kb*in->Te[i]));
	in->p[i]=in->Nv[i]*exp(((tp)*q)/(kb*in->Th[i]));

//printf("%e %e\n",t,tp);
//getchar();
	in->mun[i]=get_n_mu(in->imat[i]);
	in->mup[i]=get_p_mu(in->imat[i]);

}
in->Vl=0.0;
in->Vr=delta_phi;
in->Vbi=delta_phi;
init_dump(in);
//getchar();
if (in->stoppoint==1) exit(0);
}

void get_initial(struct device *in)
{
my_guess(in);
return;
}

