// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <gsl/gsl_multimin.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <util.h>
#include <i.h>
#include "../../server.h"
#include "fit.h"
#include "../../dump_ctrl.h"

static int unused __attribute__((unused));

static struct fitvars fitconfig;
int uptodate=FALSE;


static double last_jv_error=0.0;
static double last_charge_error=0.0;
static double last_jv_error_light=0.0;
static double last_charge_error_light=0.0;
static double last_rbi_error=0.0;
static double last_tail_error=0.0;
static double last_total_error=0.0;
static double last_tpc_error=0.0;
static double last_tof_error=0.0;
static double last_voc_tau_error=0.0;
static double last_ce_trans_error=0.0;
static double last_stark_error=0.0;
static double last_starknp_error=0.0;
static double last_pulse_error=0.0;
static double last_pulse_voc_error=0.0;
static double last_tpv_error=0.0;
static double last_celiv_error=0.0;
static double last_vocdarkmu_error=0.0;
static double last_jvtau_error=0.0;
static double last_shg_error=0.0;

double get_rbi_error()
{
char orig_file[200];
strcpy(orig_file,"./chris/k.dat");
printf("get iv error\n");
int i;
double error=0.0;
struct istruct exp;
struct istruct iv;
int fit_len=get_file_len(orig_file)-1;
int ivexternal_len=get_file_len("./rbi.dat")-1;
inter_load(&exp,orig_file,fit_len,0,1);
inter_load(&iv,"./rbi.dat",ivexternal_len,0,3);
//inter_save(&iv,"./rodtest.dat");
//getchar();
FILE *ferror;
ferror=fopen("./rbi_error.dat","w");
//inter_print(&exp);
//getchar();
//printf("len = %d %d\n", ivexternal_len,fit_len);
//getchar();
for (i=0;i<iv.len;i++)
{
if (iv.data[i]<0) iv.data[i]=0.0;
iv.data[i]=sqrt(iv.data[i]*iv.data[i]);

}
double max=0.0;
for (i=0;i<exp.len;i++)
{
//printf("%e\n",exp.data[i]);
exp.data[i]=sqrt(exp.data[i]*exp.data[i]);
if (exp.data[i]>max) max=exp.data[i];
}

for (i=0;i<iv.len;i++)
{
if (iv.data[i]>max) max=iv.data[i];
}

for (i=0;i<iv.len;i++)
{
iv.data[i]/=max;
//printf("%e\n",iv.data[i]);
}

for (i=0;i<exp.len;i++)
{
exp.data[i]/=max;
//printf("%e\n",exp.data[i]);
}
//getchar();
double start=exp.x[0];
if (iv.x[0]>start) start=iv.x[0];
double stop=exp.x[exp.len-1];
double points=0.0;
double dx=(stop-start)/1000;
double pos=start;
double etemp=0.0;
do
{
etemp=fitconfig.rbi_fitmul*fabs(inter_get(&iv,pos)-inter_get(&exp,pos));
fprintf(ferror,"%e %e %e %e\n",pos,etemp,inter_get(&iv,pos),inter_get(&exp,pos));
error+=etemp;
points+=1.0;

pos+=dx;
//printf("%e\n",error);
}while(pos<stop);

fclose(ferror);
inter_free(&exp);
inter_free(&iv);









//printf("return = %e\n",error);
//getchar();
return error/points;
}

double get_all_error(struct fitvars *myfit)
{
int i;
double local_error=0.0;

if (myfit->dotimece==TRUE)
{
last_charge_error=get_charge_error(myfit);
}else
if (myfit->fit_dark_ce_ss==TRUE)
{
last_charge_error=get_charge_ss_error(myfit);
printf("last_charge_error = %e\n",last_charge_error);
}else
{
last_charge_error=0.0;
}


if (myfit->fit_light_ce==TRUE)
{
last_charge_error_light=get_charge_light_ss_error(myfit);
printf("last_charge_error_light = %e\n",last_charge_error_light);
}else
{
last_charge_error_light=0.0;
}

last_tof_error=0.0;
for (i=0;i<myfit->tofmax;i++)
{
local_error=get_tof_error(myfit,i);
last_tof_error+=local_error;
printf("last_tof_error = %e\n",local_error);

}

last_ce_trans_error=0.0;
for (i=0;i<myfit->cemax;i++)
{
	local_error=get_ce_trans_error(myfit,i);
	last_ce_trans_error+=local_error;
	printf("last_ce_trans_error = %le\n",local_error);
}

last_tpc_error=0.0;
if (myfit->tpcmax!=0)
{
	for (i=0;i<myfit->tpcmax;i++)
	{
	local_error=get_tpc_error(myfit,i);
	last_tpc_error+=local_error;
	printf("tpc_error %d = %e\n",i,local_error);

	}
	printf("total_tpc_error = %e\n",last_tpc_error);
}

last_stark_error=0.0;

if (myfit->stark_max!=0)
{
	//if (myfit->stark_max==1)
	//{
	//	printf("1 is not allowed as we need to substract 0 V from the transient\n");
	//	exit(0);
	//}
	double save_error[10];
	for (i=0;i<myfit->stark_max;i++)
	{
	save_error[i]=get_stark_error(myfit,i);
	last_stark_error+=save_error[i];

	}

	for (i=0;i<myfit->stark_max;i++)
	{
	printf("stark_error %d = %e (%.2f)\n",i,save_error[i],(save_error[i]/last_stark_error)*100.0);

	}

	printf("total_stark_error = %e\n",last_stark_error);
}

last_shg_error=0.0;

if (myfit->shg_max!=0)
{

	for (i=0;i<myfit->shg_max;i++)
	{
	local_error=get_shg_error(myfit,i);

	last_shg_error+=local_error;
	printf("shg_error %d = %e\n",i,local_error);

	}
	printf("total_shg_error = %e\n",last_shg_error);
}

last_pulse_error=0.0;

if (myfit->pulse_max!=0)
{
	for (i=0;i<myfit->pulse_max;i++)
	{
	local_error=get_pulse_error(myfit,i);
	last_pulse_error+=local_error;
	printf("pulse_error %d = %le\n",i,local_error);

	}
	printf("total_pulse_error = %le\n",last_pulse_error);
}

last_pulse_voc_error=0.0;

if (myfit->pulse_voc_max!=0)
{
	for (i=0;i<myfit->pulse_voc_max;i++)
	{
		local_error=get_pulse_voc_error(myfit,i);
		last_pulse_voc_error+=local_error;
		printf("pulse_voc_error %d = %le\n",i,local_error);
	}
	printf("total_pulse_voc_error = %le\n",last_pulse_voc_error);
}

last_jv_error=0.0;
if (myfit->fit_jv!=0)
{
	for (i=0;i<myfit->fit_jv;i++)
	{
		local_error=get_jv_error(myfit,i);
		last_jv_error+=local_error;
		printf("last_jv_error %d = %le\n",i,local_error);
	}

	printf("total_last_jv_error = %le\n",last_jv_error);
}

last_starknp_error=0.0;
if (myfit->starknp_max!=0)
{
	for (i=0;i<myfit->starknp_max;i++)
	{
	local_error=get_starknp_error(myfit,i);
	last_starknp_error+=local_error;
	printf("starknp_error %d = %e\n",i,local_error);

	}
	printf("total_starknp_error = %e\n",last_starknp_error);
}

last_celiv_error=0.0;
if (myfit->celiv_max!=0)
{
	for (i=0;i<myfit->celiv_max;i++)
	{
	local_error=get_celiv_error(myfit,i);
	last_celiv_error+=local_error;
	printf("celiv_error %d = %e\n",i,local_error);

	}
	printf("total_celiv_error = %e\n",last_celiv_error);
}

last_tpv_error=0.0;
if (myfit->tpv_max!=0)
{
	for (i=0;i<myfit->tpv_max;i++)
	{
	local_error=get_tpv_error(myfit,i);
	last_tpv_error+=local_error;
	printf("tpv_error %d = %le\n",i,local_error);

	}
	printf("total_tpv_error = %le\n",last_tpv_error);
}

if (myfit->fit_rbi==TRUE)
{
last_rbi_error=get_rbi_error();
printf("last_rbi_error = %e\n",last_rbi_error);
}
else
{
last_rbi_error=0.0;
}

if (myfit->fit_jv>2)
{
last_voc_tau_error=get_voc_tau_error(myfit);
printf("last_voc_tau_error = %e\n",last_voc_tau_error);
}
else
{
last_voc_tau_error=0.0;
}

last_tail_error=get_tail_error(myfit);

if (myfit->fit_vocdarkmu==TRUE)
{
last_vocdarkmu_error=get_vocdarkmu(myfit);
printf("last_vocdarkmu_error = %e\n",last_vocdarkmu_error);
}
else
{
last_vocdarkmu_error=0.0;
}

if (myfit->fit_jvtau==TRUE)
{
last_jvtau_error=get_jvtau_error(myfit);
printf("last_jvtau_error = %e\n",last_jvtau_error);
}
else
{
last_jvtau_error=0.0;
}



printf("last_general_error = %e\n",last_tail_error);

last_total_error=last_jv_error+last_charge_error+last_rbi_error+last_jv_error_light+last_charge_error_light+last_tail_error+last_tpc_error+last_tof_error+last_voc_tau_error+last_ce_trans_error+last_stark_error+last_starknp_error+last_tpv_error+last_pulse_error+last_pulse_voc_error+last_celiv_error+last_vocdarkmu_error+last_jvtau_error+last_shg_error;

printf("total error = %e\n",last_total_error);
#ifndef mindump
FILE *fiterror=fopen("./fiterror.dat","w");
fprintf(fiterror,"%le\n",last_total_error);
fclose(fiterror);
#endif
return last_total_error;
}

void duplicate()
{
int from;
double val;
char orig_file[100];
char dest_file[100];
int orig_pos;
int dest_pos;
char temp[100];
char token1[100];
char token2[100];
if (get_dump_status(dump_iodump)==TRUE) printf("Duplicate\n");
FILE *file;
if ((file=fopen("./duplicate.inp","r"))==NULL)
{
	printf("No duplicate.inp file\n");
	exit(0);
}

do
{
	unused=fscanf(file,"%s",orig_file);
	if (strcmp(orig_file,"#end")==0) break;

	unused=fscanf(file,"%s",token1);
	unused=fscanf(file,"%s",dest_file);
	unused=fscanf(file,"%s",&token2);

	if (orig_file[0]!='#')
	{
		orig_pos=config_search_token(orig_file,token1);
		val=read_value(orig_file,0,orig_pos);
		dest_pos=config_search_token(dest_file,token2);
		edit_file(dest_file,"",dest_pos,val);
	}
}while(!feof(file));
fclose(file);
if (get_dump_status(dump_iodump)==TRUE) printf("Duplicate finished\n");
}

double my_f (const gsl_vector *v, void *params)
{
if (get_dump_status(dump_iodump)==TRUE) printf("my_f\n");
double val;
double e0;
int fitcount=0;

for (fitcount=0;fitcount<fitconfig.fitvars;fitcount++)
{
	if (get_dump_status(dump_iodump)==TRUE) printf("fitcount loop %d/%d \n",fitcount,fitconfig.fitvars);
	val = gsl_vector_get(v, fitcount);
	edit_file(fitconfig.fit_file[fitcount],"",fitconfig.fit_pos[fitcount],val);
}

if (get_dump_status(dump_iodump)==TRUE) printf("Exited fitcount loop\n");

duplicate();
e0=fit_run_sims(&fitconfig,&globalserver);


if (get_dump_status(dump_iodump)==TRUE) printf("exit my_f\n");
return e0;
}



void fit_dump_log(int iter,double error,double size,gsl_vector *old_vector,gsl_vector *old_shift,gsl_vector *ss,gsl_vector *x)
{
FILE *out;
if (get_dump_status(dump_iodump)==TRUE) printf("fit_dump_log\n");

gsl_vector_sub (old_vector, x);
gsl_vector_sub (old_shift, ss);
out=fopen("./fitlog.dat","a");
printf ("%d %le %le %le\n", iter, error, last_total_error,size);
fprintf (out,"%d %le %le\n", iter, error, size);
fclose(out);

out=fopen("./fit_speed.dat","a");
fprintf (out,"%d %le %le\n", iter, server_get_jobs_per_s(),server_get_odes_per_s());
fclose(out);

if (get_dump_status(dump_iodump)==TRUE) printf("exit fit_dump_log\n");

}

void do_copy()
{
FILE *file;
char path[1000];
char command[1000];

	if ((file=fopen("./rsyncdir.inp","r"))==NULL)
	{
		printf("No rsyncdir.inp file so not copying\n");
		exit(0);
	}else
	{

		unused=fscanf(file,"%s",path);
		fclose(file);
		sprintf(command,"cp ./*.dat ./*.inp %s\n",path);

		printf("Running command %s\n",command);
		unused=system(command);

	}


}

int fit_now()
{
if (get_dump_status(dump_iodump)==TRUE) printf("fit_now\n");
double val=0.0;
double par[5] = {1.0, 2.0, 10.0, 20.0, 30.0};
size_t iter = 0;
int status;
int item=0;
double size;
int fitcount=0;
int loop=0;
double shift;
int cont=TRUE;

fit_read_config(&fitconfig);

shift=fitconfig.simplexmul;
item=fitconfig.fitvars;


gsl_vector *x;
gsl_vector *ss;

gsl_vector *old_vector;
gsl_vector *old_shift;

x = gsl_vector_alloc (fitconfig.fitvars);
ss = gsl_vector_alloc (fitconfig.fitvars);
old_vector = gsl_vector_alloc (fitconfig.fitvars);
old_shift = gsl_vector_alloc (fitconfig.fitvars);

for (fitcount=0;fitcount<fitconfig.fitvars;fitcount++)
{
	val=read_value(fitconfig.fit_file[fitcount],0,fitconfig.fit_pos[fitcount]);
	gsl_vector_set (x, fitcount, val);
	gsl_vector_set (ss, fitcount, fabs(val*shift));
}


	const gsl_multimin_fminimizer_type *T =
	gsl_multimin_fminimizer_nmsimplex;//gsl_multimin_fminimizer_nmsimplex;
	gsl_multimin_function minex_func;
	gsl_multimin_fminimizer *s = NULL;


	/* Initialize method and iterate */
	minex_func.n = item;
	minex_func.f = my_f;
	minex_func.params = par;

	s = gsl_multimin_fminimizer_alloc (T, item);
	gsl_multimin_fminimizer_set (s, &minex_func, x, ss);
	double last_error=0.0;
	int length=0;
	int stop=FALSE;
	int dump_count=0;
	do
	{
		if (get_dump_status(dump_iodump)==TRUE)
		{
			printf("call gsl_multimin_fminimizer_iterate\n");
		}
		status = gsl_multimin_fminimizer_iterate(s);
		if (get_dump_status(dump_iodump)==TRUE)
		{
			printf("return from gsl_multimin_fminimizer_iterate\n");
		}

		if (status)
		{
			printf("Exited due to status error\n");
			break;
		}
		size = gsl_multimin_fminimizer_size (s);
		status = gsl_multimin_test_size (size, 1e-8);

		if (status == GSL_SUCCESS)
		{
			printf ("converged to minimum at\n");
		}

		fit_dump_log(iter,s->fval, size,old_vector,old_shift,ss,s->x);

		if (loop>fitconfig.simplexreset)
		{
			if (length>fitconfig.fitvars)
			{
				if (get_dump_status(dump_iodump)==TRUE) printf("one\n");
				stop=TRUE;
			}else
			{
				if (get_dump_status(dump_iodump)==TRUE) printf("two\n");
				loop-=fitconfig.fitvars*2;
			}
		}

		if (last_error==s->fval)
		{
			length++;
		}else
		{
			length=0;
		}

		if (status != GSL_CONTINUE) stop=TRUE;

		if ((dump_count>200)||(stop==TRUE))
		{
			if (get_dump_status(dump_iodump)==TRUE) printf("Calling dump\n");
			do_copy();
			dump_count=0;
		}

		if (get_dump_status(dump_iodump)==TRUE)
		{
			printf("Loop=%d dump_count=%d ittr=%d last_error=%e\n",loop,dump_count,iter,s->fval);
		}

		loop++;
		dump_count++;
		iter++;
		last_error=s->fval;

	}while (stop==FALSE);

	cont=TRUE;

if (get_dump_status(dump_iodump)==TRUE) printf("freeing fit memory\n");
gsl_multimin_fminimizer_free (s);
gsl_vector_free (old_vector);
gsl_vector_free (old_shift);
gsl_vector_free(x);
gsl_vector_free(ss);
if (get_dump_status(dump_iodump)==TRUE) printf("fit_now_exit\n");
return cont;
}
