// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../slice.h"
#include "../../dump.h"
#include "../../ntricks.h"
#include "../../gui_hooks.h"
#include "photokit.h"

static int unused __attribute__((unused));

struct photokit photokit_config;


void sim_photokit(struct device *in)
{
gui_start();
photokit_load_config(&photokit_config,in);
//<clean>

double stop_time=(1.0/photokit_config.photokit_modulation_fx)*photokit_config.photokit_n;

in->dt=stop_time/photokit_config.photokit_points;



in->go_time=FALSE;

in->externalv=photokit_config.photokit_Vexternal;
printf("Running photokit. %e\n",in->externalv);
int ittr=0;

int step=0;
light_solve_and_update(in,&(in->mylight),photokit_config.photokit_sun,0.0);

sim_externalv(in,photokit_config.photokit_Vexternal);
remesh_reset(in,in->Vapplied);

in->Vapplied=photokit_config.photokit_Vexternal;
double V=in->Vapplied;

light_solve_and_update(in,&(in->mylight),photokit_config.photokit_sun,0.0);

newton_photokit(in,V,FALSE);
device_timestep(in);
newton_photokit(in,V,FALSE);
device_timestep(in);
in->time=0.0;

time_init(in);

device_timestep(in);

in->go_time=TRUE;


FILE *photokit_out=fopen("./photokit_j.dat","w");
fclose(photokit_out);

photokit_out=fopen("./photokit_v.dat","w");
fclose(photokit_out);

photokit_out=fopen("./photokit_nleft.dat","w");
fclose(photokit_out);

photokit_out=fopen("./photokit_rtot.dat","w");
fclose(photokit_out);


dump_dynamic_init(in);

struct istruct out_i;
inter_init(&out_i);

struct istruct out_v;
inter_init(&out_v);

struct istruct out_mue;
inter_init(&out_mue);

struct istruct out_muh;
inter_init(&out_muh);

struct istruct out_mue_c;
inter_init(&out_mue_c);

struct istruct out_muh_c;
inter_init(&out_muh_c);

struct istruct out_G;
inter_init(&out_G);

struct istruct out_psun;
inter_init(&out_psun);

reset_npinit(in);
double Plight=0.0;
do
{

	Plight=photokit_config.photokit_sun+photokit_config.photokit_modulation_max*sin(2*pi*photokit_config.photokit_modulation_fx*in->time);
	light_solve_and_update(in,&(in->mylight),Plight,0.0);
	dump_dynamic_add_data(in);

	double i0=newton_photokit(in,V,TRUE);


	if (get_dump_status(dump_print_text)==TRUE)
	{
		printf("photokit time=%e step=%d %.1e ",in->time,step,in->last_error);
		printf("Vtot=%lf current = %e mA (%e A/m^2)\n",V,get_I(in)/1e-3,get_J(in));
	}

	ittr++;

	slice_check(in);

	plot_now(in,"photokit.plot");

	inter_append(&out_i,in->time,i0);
	inter_append(&out_v,in->time,V);
	inter_append(&out_mue,in->time,get_avg_mue(in));
	inter_append(&out_muh,in->time,get_avg_muh(in));
	inter_append(&out_mue_c,in->time,in->mun[0]);
	inter_append(&out_muh_c,in->time,in->mup[0]);
	inter_append(&out_G,in->time,in->Gn[0]);
	inter_append(&out_psun,in->time,Plight);
	device_timestep(in);
	step++;

	if (in->time>(stop_time)) break;



	if (in->stop==TRUE)
	{
		break;

	}


}while(1);
char outpath[1000];
struct istruct out_photokit;

dump_dynamic_save(in);
dump_dynamic_free(in);

inter_copy(&out_photokit,&out_i,TRUE);
inter_mul(&out_photokit,-1.0);
double avg=inter_avg(&out_photokit);
inter_sub_double(&out_photokit,avg);

sprintf(outpath,"%s%s",in->outputpath,"photokit_test.dat");
inter_save(&out_photokit,outpath);


double real=0.0;
double imag=0.0;

struct istruct peaks_i;
struct istruct peaks_sun;
inter_init(&peaks_i);
inter_init(&peaks_sun);

inter_find_peaks(&peaks_i,&out_photokit,TRUE);
inter_save(&peaks_i,"./photokit_peaks_i.dat");

inter_find_peaks(&peaks_sun,&out_psun,TRUE);
inter_save(&peaks_sun,"./photokit_peaks_sun.dat");

double dtime=(peaks_sun.x[0]-peaks_i.x[0]);
double mag=peaks_i.data[0];
real=mag*cos(2.0*3.14159*photokit_config.photokit_modulation_fx*dtime);
imag=mag*sin(2.0*3.14159*photokit_config.photokit_modulation_fx*dtime);
inter_save(&peaks_i,"min_max.dat");
inter_save(&peaks_sun,"min_max_sun.dat");
inter_free(&peaks_sun);
inter_free(&peaks_i);

inter_free(&out_photokit);

//inter_dft(&real,&imag,&out_i,photokit_config.photokit_modulation_fx);
double ang=atan(imag/real)*360/(2.0*pi);
printf("real=%le imag=%le ang=%lf\n",real,imag,ang);

FILE *out;
out=fopena(in->outputpath,"./sim_info.dat","w");
fprintf(out,"#photokit_r\n%le\n",real);
fprintf(out,"#photokit_i\n%le\n",imag);
fprintf(out,"#photokit_fx\n%le\n",photokit_config.photokit_modulation_fx);
fclose(out);


sprintf(outpath,"%s%s",in->outputpath,"photokit_i.dat");
inter_save(&out_i,outpath);

sprintf(outpath,"%s%s",in->outputpath,"photokit_i_norm_x.dat");

inter_rescale(&out_i,1.0/out_i.x[out_i.len-1], 1.0);
inter_save(&out_i,outpath);


sprintf(outpath,"%s%s",in->outputpath,"photokit_v.dat");
inter_save(&out_v,outpath);

sprintf(outpath,"%s%s",in->outputpath,"photokit_mue.dat");
inter_save(&out_mue,outpath);

sprintf(outpath,"%s%s",in->outputpath,"photokit_muh.dat");
inter_save(&out_muh,outpath);

sprintf(outpath,"%s%s",in->outputpath,"photokit_mue_c.dat");
inter_save(&out_mue_c,outpath);

sprintf(outpath,"%s%s",in->outputpath,"photokit_muh_c.dat");
inter_save(&out_muh_c,outpath);

sprintf(outpath,"%s%s",in->outputpath,"photokit_G.dat");
inter_save(&out_G,outpath);

sprintf(outpath,"%s%s",in->outputpath,"photokit_psun.dat");
inter_save(&out_psun,outpath);


in->go_time=FALSE;

inter_free(&out_G);
inter_free(&out_muh);
inter_free(&out_mue);
inter_free(&out_muh_c);
inter_free(&out_mue_c);
inter_free(&out_i);
inter_free(&out_v);
inter_free(&out_psun);

gui_stop();
}

void photokit_load_config(struct photokit *in,struct device *dev)
{
double ver=0.0;
char name[200];
FILE *config=fopena(dev->outputpath,"photokit.inp","r");

config_read_line_to_double(&(in->photokit_points),config,"#photokit_points");

config_read_line_to_double(&(in->photokit_n),config,"#photokit_n");

config_read_line_to_double(&(in->photokit_Vexternal),config,"#photokit_Vexternal");

config_read_line_to_double(&(in->Rshort_photokit),config,"#Rshort_photokit");

config_read_line_to_double(&(in->photokit_sun),config,"#photokit_sun");

config_read_line_to_double(&(in->photokit_modulation_max),config,"#photokit_modulation_max");

config_read_line_to_double(&(in->photokit_modulation_fx),config,"#photokit_modulation_fx");

config_read_line_to_double(&(ver),config,"#ver");

if (ver!=1.2)
{
ewe("File compatability problem %s\n","photokit.inp");
}

unused=fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
ewe("Problem with photokit.inp file last item read %s!\n",name);
}
fclose(config);
}
