// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include "dump.h"
#include "remesh.h"
#include "ntricks.h"
#include <math.h>


struct remesh my_mesh;

void remesh_shrink(struct device *in)
{
if (in->remesh==TRUE)
{
int i;
double pos=0.0;

my_mesh.len=in->ymeshpoints;
my_mesh.x=(double *)malloc(my_mesh.len*sizeof(double));
	for (i=0;i<my_mesh.len;i++)
	{
		my_mesh.x[i]=in->ymesh[i];
	}
in->ymeshpoints=in->newmeshsize;
double dx=in->ylen/((double)in->ymeshpoints);
my_mesh.new_dx=dx;
	for (i=0;i<in->ymeshpoints;i++)
	{
		pos+=dx/2.0;
		in->ymesh[i]=pos;
		pos+=dx/2.0;
	}
}
}

void remesh_expand_array_band(double **y,int band,struct device *in)
{
int i;
int pos;
double *temp=(double*)malloc(sizeof(double)*in->ymeshpoints);
for (i=0;i<in->ymeshpoints;i++)
{
	temp[i]=y[i][band];
}

for (i=0;i<my_mesh.len;i++)
{

	pos=in->ymeshpoints*(my_mesh.x[i]/in->ylen);
	if (pos>=in->ymeshpoints) pos=in->ymeshpoints-1;

	y[i][band]=temp[pos];
}

free(temp);
}

void remesh_expand_array(double *y,struct device *in)
{
int i;
int pos;
double *temp=(double*)malloc(sizeof(double)*in->ymeshpoints);
for (i=0;i<in->ymeshpoints;i++)
{
	temp[i]=y[i];
//printf("%le\n",temp[i]);
}
//printf("next\n");
for (i=0;i<my_mesh.len;i++)
{

	pos=in->ymeshpoints*(my_mesh.x[i]/in->ylen);
	if (pos>=in->ymeshpoints) pos=in->ymeshpoints-1;

	y[i]=temp[pos];
	//printf("%le\n",temp[pos]);
}

free(temp);
//getchar();
}

void remesh_reset(struct device *in,double voltage)
{
int i;
int band;
if (in->remesh==TRUE)
{
newton_push_state(in);
in->max_electrical_itt=1000;
in->newton_min_itt=10;
in->electrical_clamp=4.0;

	remesh_expand_array(in->phi,in);

	remesh_expand_array(in->x,in);
	remesh_expand_array(in->xp,in);


	for (band=0;band<in->srh_bands;band++)
	{
		remesh_expand_array_band(in->xt,band,in);
		remesh_expand_array_band(in->xpt,band,in);
	}


for (i=0;i<my_mesh.len;i++)
{
	in->ymesh[i]=my_mesh.x[i];
}


in->ymeshpoints=my_mesh.len;
my_mesh.len=0;
update_arrays(in);

sim_externalv(in,voltage);
newton_pop_state(in);

free(my_mesh.x);
}
}
