// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../dump.h"
#include "tpc.h"

double sim_tpc_step(struct device *in)
{
double clamp=0.1;
double step=0.01;
double e0;
double e1;
double i0;
double i1;
double deriv;
double C=in->area*epsilon0*in->epsilonr[0]/in->ylen;
//double R1=in->Rcontact+in->Rshort;
//double R2=in->Rshunt;
solve_all(in);
i0=get_I(in);
//getchar();
double igen=(i0+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt);

e0=fabs(igen*in->Rcontact+in->Vapplied-tpc_config.tpc_externalv);
in->Vapplied+=step;
solve_all(in);
i1=get_I(in);
igen=(i1+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt);
e1=fabs(igen*in->Rcontact+in->Vapplied-tpc_config.tpc_externalv);
printf("%le\n",e1);
deriv=(e1-e0)/step;
step=-e1/deriv;
//step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
int count=0;
int max=1000;
do
{
e0=e1;
solve_all(in);
igen=(i1+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt);
e1=fabs(igen*in->Rcontact+in->Vapplied-tpc_config.tpc_externalv);
printf("%le %le %le %le\n",igen*in->Rcontact,in->Vapplied,-tpc_config.tpc_externalv,e1);
deriv=(e1-e0)/step;
step=-e1/deriv;
//double clamp=0.01;
//if (e1<clamp) clamp=e1/100.0;
//step=step/(1.0+fabs(step/clamp));
step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
//printf("ce error = %e %e\n",in->Vapplied,e1);
if (count>max) break;
count++;
}while(e1>1e-5);
//printf("result = %e %e %e %e %e\n",i1,in->Vapplied/Rshort,C*(in->Vapplied-in->Vapplied_last)/in->dt,i1+in->Vapplied/Rshort+C*(in->Vapplied-in->Vapplied_last)/in->dt);
double ret=in->Vapplied-C*(i1-in->Ilast)/in->dt;
return ret;
}

void newton_aux_tpc(struct device *in,double V,double* i,double* didv,double* didphi,double* didxil,double* didxipl,double* didphir,double* didxir,double* didxipr)
{
//<clean>
double C=0.0;
double L=0.0;
double i0=*i;
double didv0=*didv;
double didphi0=*didphi;
double didxil0=*didxil;
double didxipl0=*didxipl;
double didphir0=*didphir;
double didxir0=*didxir;
double didxipr0=*didxipr;
double itot=i0+C*(V-in->Vapplied_last)/in->dt+V/in->Rshunt;
double ditotdv=didv0+C*(1.0)/in->dt+1.0/in->Rshunt;
C=in->C;
L=tpc_config.tpc_L;
*i=tpc_config.tpc_power_suply+L*(itot-in->ilast)/in->dt-(V+(in->Rcontact+tpc_config.Rtpc_short)*(itot));
*didv=L*(ditotdv)/in->dt-(1.0+(in->Rcontact+tpc_config.Rtpc_short)*(ditotdv));
*didphi=L*(didphi0)/in->dt-((in->Rcontact+tpc_config.Rtpc_short)*(didphi0));
*didxil=L*(didxil0)/in->dt-((in->Rcontact+tpc_config.Rtpc_short)*(didxil0));
*didxipl=L*(didxipl0)/in->dt-((in->Rcontact+tpc_config.Rtpc_short)*(didxipl0));
*didphir=L*(didphir0)/in->dt-((in->Rcontact+tpc_config.Rtpc_short)*(didphir0));
*didxir=L*(didxir0)/in->dt-((in->Rcontact+tpc_config.Rtpc_short)*(didxir0));
*didxipr=L*(didxipr0)/in->dt-((in->Rcontact+tpc_config.Rtpc_short)*(didxipr0));
//</clean>
return;
}

double newton_sim_tpc_all(struct device *in)
{
in->newton_cur=TRUE;
solver_realloc(in);
in->newton_aux=&newton_aux_tpc;
solve_all(in);//in->C*(in->Vapplied-in->Vapplied_last)/in->dt+
return get_I(in)+in->C*(in->Vapplied-in->Vapplied_last)+in->Vapplied/in->Rshunt;
}

void newton_sim_tpc(struct device *in)
{
in->newton_cur=FALSE;
solver_realloc(in);
solve_all(in);
}

